Attribute VB_Name = "MHelloWorld"
' *************************************************************************
'  Copyright 2000 Karl E. Peterson
'  All Rights Reserved, http://www.mvps.org/vb
' *************************************************************************
'  Based heavily on Charles Petzold's classic HELLOWIN program.
'  <cite>Programming Windows, Fifth Edition</cite>, MS Press.
'  http://mspress.microsoft.com/prod/books/2344.htm
' *************************************************************************
'  You are free to use this code within your own applications, but you
'  are expressly forbidden from selling or otherwise distributing this
'  source code, non-compiled, without prior written consent.
' *************************************************************************
Option Explicit

' Required Win32 API functions
Private Declare Function RegisterClassEx Lib "user32" Alias "RegisterClassExA" (pcWndClassEx As WNDCLASSEX) As Long
Private Declare Function UnregisterClass Lib "user32" Alias "UnregisterClassA" (ByVal lpClassName As String, ByVal hInstance As Long) As Long
Private Declare Function DefWindowProc Lib "user32" Alias "DefWindowProcA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function CreateWindowEx Lib "user32" Alias "CreateWindowExA" (ByVal dwExStyle As Long, ByVal lpClassName As String, ByVal lpWindowName As String, ByVal dwStyle As Long, ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hWndParent As Long, ByVal hMenu As Long, ByVal hInstance As Long, lpParam As Any) As Long
Private Declare Sub GetStartupInfo Lib "kernel32" Alias "GetStartupInfoA" (lpStartupInfo As STARTUPINFO)
Private Declare Function ShowWindow Lib "user32" (ByVal hWnd As Long, ByVal nCmdShow As Long) As Long
Private Declare Function UpdateWindow Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function DestroyWindow Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function GetMessage Lib "user32" Alias "GetMessageA" (lpMsg As msg, ByVal hWnd As Long, ByVal wMsgFilterMin As Long, ByVal wMsgFilterMax As Long) As Long
Private Declare Function TranslateMessage Lib "user32" (lpMsg As msg) As Long
Private Declare Function DispatchMessage Lib "user32" Alias "DispatchMessageA" (lpMsg As msg) As Long
Private Declare Function PostMessage Lib "user32" Alias "PostMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Sub PostQuitMessage Lib "user32" (ByVal nExitCode As Long)
Private Declare Sub ExitProcess Lib "kernel32" (ByVal uExitCode As Long)
Private Declare Function LoadCursor Lib "user32" Alias "LoadCursorA" (ByVal hInstance As Long, ByVal lpCursorName As String) As Long
Private Declare Function LoadIcon Lib "user32" Alias "LoadIconA" (ByVal hInstance As Long, ByVal lpIconName As String) As Long
Private Declare Function GetStockObject Lib "gdi32" (ByVal nIndex As Long) As Long

' Painting/Multimedia APIs
Private Declare Function BeginPaint Lib "user32" (ByVal hWnd As Long, lpPaint As PAINTSTRUCT) As Long
Private Declare Function EndPaint Lib "user32" (ByVal hWnd As Long, lpPaint As PAINTSTRUCT) As Long
Private Declare Function GetClientRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT) As Long
Private Declare Function CreateSolidBrush Lib "gdi32" (ByVal crColor As Long) As Long
Private Declare Function FillRect Lib "user32" (ByVal hDC As Long, lpRect As RECT, ByVal hBrush As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function PlaySound Lib "winmm.dll" Alias "PlaySoundA" (ByVal lpszName As String, ByVal hModule As Long, ByVal dwFlags As Long) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)

' Window class extended definition
Private Type WNDCLASSEX
   cbSize As Long
   Style As Long
   lpfnWndProc As Long
   cbClsExtra As Long
   cbWndExtra As Long
   hInstance As Long
   hIcon As Long
   hCursor As Long
   hbrBackground As Long
   lpszMenuName As Long
   lpszClassName As String
   hIconSm As Long
End Type

' Startup infomation used to position new window
Private Type STARTUPINFO
   cb As Long
   lpReserved As Long
   lpDesktop As Long
   lpTitle As Long
   dwX As Long
   dwY As Long
   dwXSize As Long
   dwYSize As Long
   dwXCountChars As Long
   dwYCountChars As Long
   dwFillAttribute As Long
   dwFlags As Long
   wShowWindow As Integer
   cbReserved2 As Integer
   lpReserved2 As Long
   hStdInput As Long
   hStdOutput As Long
   hStdError As Long
End Type

' Message prototypes
Private Type POINTAPI
   x As Long
   y As Long
End Type

Private Type msg
   hWnd As Long
   message As Long
   wParam As Long
   lParam As Long
   time As Long
   pt As POINTAPI
End Type

' Paint-related structures
Private Type RECT
   Left As Long
   Top As Long
   Right As Long
   Bottom As Long
End Type

Private Type PAINTSTRUCT
   hDC As Long
   fErase As Long
   rcPaint As RECT
   fRestore As Long
   fIncUpdate As Long
   rgbReserved(1 To 32) As Byte
End Type

' Window messages
Private Const WM_CREATE = &H1
Private Const WM_DESTROY = &H2
Private Const WM_CLOSE = &H10
Private Const WM_PAINT = &HF
Private Const WM_KEYDOWN = &H100
Private Const WM_KEYUP = &H101
Private Const WM_CHAR = &H102

' Class styles
Private Const CS_VREDRAW = &H1
Private Const CS_HREDRAW = &H2
Private Const CS_CLASSDC = &H40

' CreateWindow constants
Private Const CW_USEDEFAULT = &H80000000

' Standard Cursor IDs
Private Const IDC_ARROW = 32512&

' Stock Logical Objects
Private Const WHITE_BRUSH = 0
Private Const LTGRAY_BRUSH = 1
Private Const GRAY_BRUSH = 2
Private Const DKGRAY_BRUSH = 3
Private Const BLACK_BRUSH = 4
Private Const NULL_BRUSH = 5

' Startup flags
Private Const STARTF_USESHOWWINDOW = &H1
Private Const STARTF_USESIZE = &H2
Private Const STARTF_USEPOSITION = &H4
Private Const STARTF_USECOUNTCHARS = &H8
Private Const STARTF_USEFILLATTRIBUTE = &H10
Private Const STARTF_RUNFULLSCREEN = &H20        '  ignored for non-x86 platforms
Private Const STARTF_FORCEONFEEDBACK = &H40
Private Const STARTF_FORCEOFFFEEDBACK = &H80
Private Const STARTF_USESTDHANDLES = &H100
Private Const STARTF_USEHOTKEY = &H200

' Sound constants
Private Const SND_ASYNC = &H1         ' play asynchronously
Private Const SND_ALIAS = &H10000     ' name is a WIN.INI [sounds] entry
Private Const SND_FILENAME = &H20000  ' name is a file name

' Window Styles
Private Const WS_OVERLAPPED = &H0&
Private Const WS_POPUP = &H80000000
Private Const WS_CHILD = &H40000000
Private Const WS_MINIMIZE = &H20000000
Private Const WS_VISIBLE = &H10000000
Private Const WS_DISABLED = &H8000000
Private Const WS_CLIPSIBLINGS = &H4000000
Private Const WS_CLIPCHILDREN = &H2000000
Private Const WS_MAXIMIZE = &H1000000
Private Const WS_CAPTION = &HC00000                  '  WS_BORDER Or WS_DLGFRAME
Private Const WS_BORDER = &H800000
Private Const WS_DLGFRAME = &H400000
Private Const WS_VSCROLL = &H200000
Private Const WS_HSCROLL = &H100000
Private Const WS_SYSMENU = &H80000
Private Const WS_THICKFRAME = &H40000
Private Const WS_GROUP = &H20000
Private Const WS_TABSTOP = &H10000
Private Const WS_MINIMIZEBOX = &H20000
Private Const WS_MAXIMIZEBOX = &H10000

'   Common Window Styles
Private Const WS_OVERLAPPEDWINDOW = (WS_OVERLAPPED Or WS_CAPTION Or WS_SYSMENU Or WS_THICKFRAME Or WS_MINIMIZEBOX Or WS_MAXIMIZEBOX)
Private Const WS_TILEDWINDOW = WS_OVERLAPPEDWINDOW
Private Const WS_POPUPWINDOW = (WS_POPUP Or WS_BORDER Or WS_SYSMENU)
Private Const WS_CHILDWINDOW = WS_CHILD

' Extended Window Styles
Private Const WS_EX_DLGMODALFRAME = &H1
Private Const WS_EX_NOPARENTNOTIFY = &H4
Private Const WS_EX_TOPMOST = &H8
Private Const WS_EX_ACCEPTFILES = &H10
Private Const WS_EX_TRANSPARENT = &H20
Private Const WS_EX_MDICHILD = &H40
Private Const WS_EX_TOOLWINDOW = &H80
Private Const WS_EX_WINDOWEDGE = &H100
Private Const WS_EX_CLIENTEDGE = &H200
Private Const WS_EX_CONTEXTHELP = &H400
Private Const WS_EX_RIGHT = &H1000
Private Const WS_EX_LEFT = &H0
Private Const WS_EX_RTLREADING = &H2000
Private Const WS_EX_LTRREADING = &H0
Private Const WS_EX_LEFTSCROLLBAR = &H4000
Private Const WS_EX_RIGHTSCROLLBAR = &H0
Private Const WS_EX_CONTROLPARENT = &H10000
Private Const WS_EX_STATICEDGE = &H20000
Private Const WS_EX_APPWINDOW = &H40000

' Member variables
Private m_ExitCode As Long

' Private constants
Private Const WndClass As String = "HelloWorldClass"
Private Const AppIcon As String = "APPICON"
Private Const AppCursor As String = "#101"

' *****************************************************
'  VB 's equivalent of WinMain, such as it is.
' *****************************************************
Public Sub Main()
   Dim hWnd As Long
   Dim cls As WNDCLASSEX
   Dim si As STARTUPINFO
   Dim nStyles As Long
   Dim nStylesEx As Long
   Dim nMsg As msg

   ' Define and register a custom window class.
   ' Icon and cursor settings are optional, and
   ' will only work from compiled app anyway.
   ' Using a NULL brush will prevent background
   ' from auto-clearing on every WM_PAINT.
   With cls
      .cbSize = Len(cls)
      .Style = CS_CLASSDC Or CS_HREDRAW Or CS_VREDRAW
      .lpfnWndProc = FcnPtr(AddressOf WindowProc)
      .hInstance = App.hInstance
      .hIcon = LoadIcon(App.hInstance, AppIcon)
      .hCursor = LoadCursor(App.hInstance, AppCursor)
      .hbrBackground = GetStockObject(NULL_BRUSH)
      .lpszClassName = WndClass
   End With
   Call RegisterClassEx(cls)

   ' Set style bits
   nStyles = WS_OVERLAPPEDWINDOW
   nStylesEx = WS_EX_NOPARENTNOTIFY

   ' Get startup information for positioning window,
   ' optionally using x/y/xsize/ysize values.
   si.cb = Len(si)
   Call GetStartupInfo(si)
   
   ' If you would like to see how to get at the
   ' string data in the STARTUPINFO structure,
   ' download http://www.mvps.org/vb/code/HexDump.zip
   ' Debug.Print HexDump(VarPtr(si), 256)
   ' Debug.Print PointerToStringA(si.lpDesktop)
   ' Debug.Print PointerToStringA(si.lpTitle)

   ' Create a message window based on new
   ' custom window class.
   hWnd = CreateWindowEx(nStylesEx, WndClass, _
            "Hello World, from VB!", nStyles, _
            CW_USEDEFAULT, CW_USEDEFAULT, _
            CW_USEDEFAULT, CW_USEDEFAULT, _
            0&, 0&, App.hInstance, 0&)

   ' Show window as requested, and paint.
   Call ShowWindow(hWnd, si.wShowWindow)
   Call UpdateWindow(hWnd)

   ' This could happen at any time, anywhere,
   ' but for the demo we'll just set the exit
   ' code to whatever was on command line.
   m_ExitCode = CLng(Val(Command$))

   ' Enter message loop, dispatching to WndProc
   ' as each new message arrives. Exit loop on
   ' WM_QUIT, posted from WM_DESTROY.
   Do While GetMessage(nMsg, 0&, 0&, 0&)
      TranslateMessage nMsg
      DispatchMessage nMsg
   Loop

   ' Destroy custom window class.
   Call UnregisterClass(WndClass, App.hInstance)

   ' Return appropriate exit code, but *only*
   ' if running from EXE, else IDE exits too.
   ' App *must* be compiled to native code to
   ' avoid a nasty shutdown GPF in runtime!
   If Compiled Then
      Call ExitProcess(nMsg.wParam)
   End If
End Sub

' *****************************************************
'  WndProc: Where it all happens.
' *****************************************************
Private Function WindowProc(ByVal hWnd As Long, ByVal msg As Long, ByVal wp As Long, ByVal lp As Long) As Long
   ' Filter for messages of interest, but
   ' allow default processing for most.
   Select Case msg
      Case WM_CREATE
         ' Acknowledge window creation!
         Call PlaySound("SystemStart", 0, SND_ASYNC Or SND_ALIAS)

      Case WM_DESTROY
         ' WM_QUIT breaks out of message loop.
         Call PostQuitMessage(m_ExitCode)

      Case WM_PAINT
         ' Paint something, anything...
         Call PaintGradient(hWnd)

      Case WM_CHAR
         ' Bail if escape key was pressed.
         If wp = vbKeyEscape Then
            Call PostMessage(hWnd, WM_CLOSE, 0, 0)
         Else
            WindowProc = DefWindowProc(hWnd, msg, wp, lp)
         End If
         
      Case Else
         ' Default processing...
         WindowProc = DefWindowProc(hWnd, msg, wp, lp)
   End Select
End Function

' *****************************************************
'  Utility functions
' *****************************************************
Private Function Compiled() As Boolean
   ' Determine if running from EXE/IDE.
   On Error Resume Next
   Debug.Print 1 / 0
   Compiled = (Err.Number = 0)
End Function

Private Function FcnPtr(ByVal Whatever As Long) As Long
   ' Return whatever was passed.
   FcnPtr = Whatever
End Function

Private Sub PaintGradient(ByVal hWnd As Long, Optional ByVal Shades As Long = 64)
   Static hDC As Long
   Static ps As PAINTSTRUCT
   Static r As RECT
   Static BandHeight As Long
   Static hBrush As Long
   Static i As Integer

   ' Determine dimensions of client area.
   Call GetClientRect(hWnd, r)
   
   ' Find Height for each shade band (allowing
   ' a one pixel slop for uneven divisions).
   BandHeight = (r.Bottom \ Shades) + 1
   
   ' Adjust number of shades to value <= 256.
   If Shades > r.Bottom Then Shades = r.Bottom
   If Shades < 1 Or Shades > 256 Then Shades = 64

   ' Set topmost shade band lower dimension.
   r.Bottom = BandHeight
   
   ' Get DC handle to paint upon.
   hDC = BeginPaint(hWnd, ps)

   ' Paint shaded bands...
   For i = 255 To 0 Step -(256 \ Shades)
      ' Create appropriate brush, fill rect,
      ' and delete brush.
      hBrush = CreateSolidBrush(RGB(0, 0, i))
      Call FillRect(hDC, r, hBrush)
      Call DeleteObject(hBrush)

      ' Increment band rectangle.
      r.Top = r.Bottom
      r.Bottom = r.Top + BandHeight
   Next i

   ' Clean up
   Call EndPaint(hWnd, ps)
End Sub

