Attribute VB_Name = "MMsgSink"
' *************************************************************************
'  Copyright 2000 Karl E. Peterson
'  All Rights Reserved, http://www.mvps.org/vb
' *************************************************************************
'  You are free to use this code within your own applications, but you
'  are expressly forbidden from selling or otherwise distributing this
'  source code, non-compiled, without prior written consent.
' *************************************************************************
 Option Explicit

' Required Win32 API functions
Private Declare Function RegisterClassEx Lib "user32" Alias "RegisterClassExA" (pcWndClassEx As WNDCLASSEX) As Long
Private Declare Function UnregisterClass Lib "user32" Alias "UnregisterClassA" (ByVal lpClassName As String, ByVal hInstance As Long) As Long
Private Declare Function DefWindowProc Lib "user32" Alias "DefWindowProcA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function CreateWindowEx Lib "user32" Alias "CreateWindowExA" (ByVal dwExStyle As Long, ByVal lpClassName As String, ByVal lpWindowName As String, ByVal dwStyle As Long, ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hWndParent As Long, ByVal hMenu As Long, ByVal hInstance As Long, lpParam As Any) As Long
Private Declare Function DestroyWindow Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function IsWindow Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)

' Window messages
Private Const WM_DESTROY = &H2
Private Const WM_CLOSE = &H10
Private Const WM_QUERYENDSESSION = &H11
Private Const WM_SYSCOLORCHANGE = &H15
Private Const WM_SETTINGCHANGE = &H1A
Private Const WM_ACTIVATEAPP = &H1C
Private Const WM_FONTCHANGE = &H1D
Private Const WM_TIMECHANGE = &H1E
Private Const WM_SPOOLERSTATUS = &H2A
Private Const WM_USERCHANGED = &H54
Private Const WM_DISPLAYCHANGE = &H7E
Private Const WM_DEVICECHANGE = &H219
Private Const WM_POWERBROADCAST = &H218

' Class styles
Private Const CS_CLASSDC = &H40

' Window class extended definition
Private Type WNDCLASSEX
   cbSize As Long
   Style As Long
   lpfnWndProc As Long
   cbClsExtra As Long
   cbWndExtra As Long
   hInstance As Long
   hIcon As Long
   hCursor As Long
   hbrBackground As Long
   lpszMenuName As Long
   lpszClassName As String
   hIconSm As Long
End Type

' Extended Window Styles
Private Const WS_EX_NOPARENTNOTIFY = &H4

' Member variables
Private m_hWnd As Long

' Private constants
Private Const WndClass As String = "SysMsgSink"

' *******************************************************
'  Public Methods
' *******************************************************
Public Function SinkCreate() As Long
   Dim cls As WNDCLASSEX
   
   ' This should *not* run when another message
   ' sink is already active!
   If IsWindow(m_hWnd) = 0 Then
      ' Define and register a message window class.
      With cls
         .cbSize = Len(cls)
         .Style = CS_CLASSDC
         .lpfnWndProc = FcnPtr(AddressOf WindowProc)
         .hInstance = App.hInstance
         .lpszClassName = WndClass
      End With
      Call RegisterClassEx(cls)
      
      ' Create a message window based on new custom class.
      m_hWnd = CreateWindowEx(0&, WndClass, _
               WndClass, 0&, 0&, 0&, 0&, 0&, _
               0&, 0&, App.hInstance, 0&)
   End If
   
   ' Return handle to created window.
   SinkCreate = m_hWnd
End Function

Public Function SinkDestroy() As Long
   ' Make sure sink is still around.
   If IsWindow(m_hWnd) Then
      ' Close created window.
      Call SinkSendMessage(WM_CLOSE, 0, 0)
      ' Unregister window class.
      Call UnregisterClass(WndClass, App.hInstance)
   End If
   SinkDestroy = SinkHandle()
End Function

Public Function SinkHandle() As Long
   ' If sink is no longer around, make
   ' sure cached handle is zero'd.
   If IsWindow(m_hWnd) = 0 Then
      m_hWnd = 0
   End If
   ' Return cached handle.
   SinkHandle = m_hWnd
End Function

Public Function SinkSendMessage(ByVal msg As Long, ByVal wp As Long, ByVal lp As Long) As Long
   ' Pass message directly to created window.
   SinkSendMessage = SendMessage(m_hWnd, msg, wp, ByVal lp)
End Function

Public Function SinkSendMessageStr(ByVal msg As Long, ByVal wp As Long, ByVal lp As String) As Long
   ' Pass message directly to created window.
   SinkSendMessageStr = SendMessage(m_hWnd, msg, wp, ByVal lp)
End Function

' *******************************************************
'  Private Methods
' *******************************************************
Private Function WindowProc(ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
   ' This procedure is *meant* to be modified to suit the needs
   ' of the application at hand.  It makes sense to use just a
   ' single message sink window per application, so filter here
   ' for messages of interest, but allow default processing for
   ' most others.  All messages used here are for example purposes
   ' only.
   Select Case wMsg
      Case WM_ACTIVATEAPP
         ' We were either just activated or deactivated.
         ' Take note, react?, and return zero.
         Debug.Print "WM_ACTIVATEAPP", CBool(wParam), "Thread: &h" & Hex(lParam)
         
      Case WM_DISPLAYCHANGE
         ' User just changed display settings.
         ' Take note, react?, and return zero.
         Debug.Print "WM_DISPLAYCHANGE", _
            LoWord(lParam); "x"; HiWord(lParam); ", "; _
            wParam; "BitsPerPixel"
            
      Case WM_FONTCHANGE
         ' An application has added/removed
         ' a font from the system.
         ' Take note, react?, and return zero.
         Debug.Print "WM_FONTCHANGE"
         
      Case WM_POWERBROADCAST
         ' Sent to an application to notify it of
         ' power-management events. Numerous events
         ' may trigger this, each requiring specific
         ' returns. For demo, we'll just use default
         ' processing, and take note.
         Debug.Print "WM_POWERBROADCAST"
         WindowProc = DefWindowProc(hWnd, wMsg, wParam, lParam)

      Case WM_QUERYENDSESSION
         ' Sent when the user chooses to end the session
         ' or when an application calls the ExitWindows
         ' function. If any application returns zero, the
         ' session is not ended. For demo, we'll allow
         ' default processing, and take note.
         Debug.Print "WM_QUERYENDSESSION"
         WindowProc = DefWindowProc(hWnd, wMsg, wParam, lParam)
         
      Case WM_SETTINGCHANGE
         ' Sent when the SystemParametersInfo function
         ' changes a system-wide setting. Lots of
         ' possibilities. Take note, react?, and
         ' return zero.
         Debug.Print "WM_SETTINGCHANGE", "wFlag = &h"; Hex(wParam)
         
      Case WM_SPOOLERSTATUS
         ' IMPORTANT: See MSKB Q161734 for Windows 2000 details!
         ' http://support.microsoft.com/support/kb/articles/q161/7/34.asp
         '
         ' Sent from Windows Print Manager whenever a job
         ' is added to or removed from the Print Manager
         ' queue. Take note, react?, and return zero.
         
         Debug.Print "WM_SPOOLERSTATUS", _
            "Job Status:"; wParam, _
            "Jobs Left:"; LoWord(lParam)

      Case WM_SYSCOLORCHANGE
         ' System colors have been changed. Must be forwarded
         ' to common controls, so do default processing. If
         ' caching brushes/pens, destroy and recreate now.
         ' Take note, react?, and return zero.
         Debug.Print "WM_SYSCOLORCHANGE"
         WindowProc = DefWindowProc(hWnd, wMsg, wParam, lParam)
         
      Case WM_TIMECHANGE
         ' System time has been changed.
         ' Take note, react?, and return zero.
         Debug.Print "WM_TIMECHANGE"
         
      Case Else
         ' Default processing...
         WindowProc = DefWindowProc(hWnd, wMsg, wParam, lParam)
   End Select
End Function

Private Function FcnPtr(ByVal Whatever As Long) As Long
   ' Return whatever was passed.
   FcnPtr = Whatever
End Function

Private Function LoWord(ByVal LongIn As Long) As Integer
   Call CopyMemory(LoWord, LongIn, 2)
End Function

Private Function HiWord(ByVal LongIn As Long) As Integer
   Call CopyMemory(HiWord, ByVal (VarPtr(LongIn) + 2), 2)
End Function

Private Function MakeLong(ByVal HiWord As Integer, ByVal LoWord As Integer) As Long
   Call CopyMemory(MakeLong, LoWord, 2)
   Call CopyMemory(ByVal (VarPtr(MakeLong) + 2), HiWord, 2)
End Function

